<?php

declare(strict_types=1);

namespace Drupal\firebase_php\Service;

use Drupal\firebase_php\Enum\SendingDestinationType;
use Drupal\firebase_php\Exception\FirebasePhpInvalidArgumentException;
use Drupal\firebase_php\FirebasePhpMessagingApiInterface;
use Kreait\Firebase\Contract\Messaging;
use Kreait\Firebase\Messaging\ApnsConfig;
use Kreait\Firebase\Messaging\CloudMessage;
use Kreait\Firebase\Messaging\MulticastSendReport;
use Kreait\Firebase\Messaging\Notification;

/**
 * Provides a Drupal API for firebase-php.
 *
 * Totally optional! You can use firebase-php however you want by extending
 * FirebasePhpMessagingService, just like this API does.
 */
class FirebasePhpMessagingApi extends FirebasePhpMessagingService implements FirebasePhpMessagingApiInterface {

  /**
   * {@inheritdoc}
   */
  public function getMessaging(): Messaging {
    return $this->messaging;
  }

  /**
   * {@inheritdoc}
   */
  public function sendMessageSingleDevice(
    string $notification_title,
    string $notification_body,
    string $device_token,
    ?string $notification_url = NULL,
    ?array $data = NULL,
    ?array $apns_data = NULL,
    ?int $badge_count = NULL,
  ): array {
    $notification = $this->createNotification($notification_title, $notification_body, $notification_url);
    $message_array = $this->createMessageSingleDevice($device_token, $notification, $data);
    $message = $this->createCloudMessage($message_array, $badge_count, $apns_data);

    return $this->messaging->send($message);
  }

  /**
   * {@inheritdoc}
   */
  public function sendMessageTopic(
    string $notification_title,
    string $notification_body,
    string $topic,
    ?string $notification_url = NULL,
    ?array $data = NULL,
    ?array $apns_data = NULL,
    ?int $badge_count = NULL,
  ): array {
    $notification = $this->createNotification($notification_title, $notification_body, $notification_url);
    $message_array = $this->createMessageTopic($topic, $notification, $data);
    $message = $this->createCloudMessage($message_array, $badge_count, $apns_data);

    return $this->messaging->send($message);
  }

  /**
   * {@inheritdoc}
   */
  public function sendMessageMultipleDevices(
    string $notification_title,
    string $notification_body,
    array $device_tokens,
    ?string $notification_url = NULL,
    ?array $data = NULL,
    ?array $apns_data = NULL,
    ?int $badge_count = NULL,
  ): MulticastSendReport {
    $notification = $this->createNotification($notification_title, $notification_body, $notification_url);
    // For multicast, create a message to the first device token.
    $message_array = $this->createMessageSingleDevice($device_tokens[0], $notification, $data);
    // Then send that message to all the device tokens.
    $message = $this->createCloudMessage($message_array, $badge_count, $apns_data);
    return $this->messaging->sendMulticast($message, $device_tokens);
  }

  /**
   * Creates a notification for use in sending messages.
   *
   * All messages need a notification.
   *
   * @param string $title
   *   The title of the notification.
   * @param string $body
   *   The body of the notification.
   * @param string|null $url
   *   The optional URL for the notification.
   *
   * @return \Kreait\Firebase\Messaging\Notification
   *   The notification.
   */
  private function createNotification(string $title, string $body, ?string $url = NULL): Notification {
    return Notification::create($title, $body, $url);
  }

  /**
   * Prepares an array for CloudMessage::fromArray().
   *
   * @param \Kreait\Firebase\Messaging\Notification $notification
   *   The notification to be sent.
   * @param \Drupal\firebase_php\Enum\SendingDestinationType $destination_type
   *   The type of destination to send the message to. Used for validation.
   * @param string $destination
   *   The destination (deviceToken or topic).
   * @param array|null $data
   *   The optional message data.
   *
   * @return array
   *   An array ready for CloudMessage::fromArray().
   */
  private function createMessageArray(
    Notification $notification,
    SendingDestinationType $destination_type,
    string $destination,
    ?array $data = NULL,
  ): array {
    $message_inputs = [
      'notification' => $notification,
    ];
    if (is_array($data)) {
      $message_inputs['data'] = $data;
    }

    if (trim($destination) !== '') {
      /** @var non-empty-string $destination */
      $message_inputs[$destination_type->value] = $destination;
    }
    else {
      throw new FirebasePhpInvalidArgumentException('Either device token or topic must be provided.');
    }

    return $message_inputs;
  }

  /**
   * Creates a message for a single device.
   */
  private function createMessageSingleDevice(string $deviceToken, Notification $notification, ?array $data = NULL): array {
    return $this->createMessageArray($notification, SendingDestinationType::DeviceToken, $deviceToken, $data);
  }

  /**
   * Creates a message for a topic.
   */
  private function createMessageTopic(string $topic, Notification $notification, ?array $data = NULL): array {
    return $this->createMessageArray($notification, SendingDestinationType::Topic, $topic, $data);
  }

  /**
   * Creates a CloudMessage to be sent.
   *
   * @param array $message_inputs
   *   An array generated by $this->createMessageArray().
   * @param int|null $badge_count
   *   The optional badge count.
   * @param array|null $apns_data
   *   The optional APNS data.
   *
   * @return \Kreait\Firebase\Messaging\CloudMessage
   *   The message to be sent.
   */
  private function createCloudMessage(
    array $message_inputs,
    ?int $badge_count = NULL,
    ?array $apns_data = NULL,
  ): CloudMessage {
    $message = CloudMessage::fromArray($message_inputs);

    if (is_array($apns_data)) {
      $message = $message->withApnsConfig($apns_data);
    }

    if (is_int($badge_count) && $badge_count >= 0) {
      $message = $message->withApnsConfig(ApnsConfig::new()->withBadge($badge_count));
    }

    return $message;
  }

}
